import TransferDom from '@/components/transfer-dom';
import Spinner from '@/components/spinner';
import PullHook from '@/core/elements/pull/pull-hook';
import Pull from '@/core/elements/pull';
import { getPageClass, getPageConfig } from '@/libs/vueUtil';
import { isArryStringMatchPattern, isPromise } from '@/libs/util';
import androidInputScroll from '@/core/elements/input/androidInputScroll';
import { getRelativeOffset } from '@/libs/dom';
import animateScroll from '@/libs/animateScroll';

var PullFactory = function PullFactory () {};

PullFactory.create = function create (vm) {
  var options = {
    outerEl: vm.container,
    innerEl: vm.transformContainer
  };

  if (vm.pullDown) {
    options.pullDown = Object.assign({}, {scene: vm.scene,
      pullEl: PullFactory.getPullEl(vm, 'pull-down'),
      onStateChange: vm.handlePullStateChange,
      onMove: vm.handlePullMove},
      vm.pullDown);

    vm.$watch('pullDown', function (val) {
      vm.pull.options.pullDown = val;
    });
  }

  if (vm.pullUp) {
    options.pullUp = Object.assign({}, {scene: vm.scene,
      pullEl: PullFactory.getPullEl(vm, 'pull-up'),
      onStateChange: vm.handlePullStateChange,
      onMove: vm.handlePullMove},
      vm.pullUp);

    vm.$watch('pullUp', function (val) {
      vm.pull.options.pullUp = val;
    });
  }

  if (Object.keys(options).length >= 3) {
    return new Pull(options)
  }
};
PullFactory.getPullEl = function getPullEl (vm, slotName) {
  if (vm.$slots[slotName]) {
    return vm.$slots[slotName][0].elm
  } else {
    return vm.$el.querySelector(("." + slotName + "-content"))
  }
};

/**
 * 前置条件：
 * Android键盘弹起/收起后，文本框仍然处于焦点状态
 * 解决问题：
 * 1. 键盘收起状态：点击文本框，键盘再次弹出
 * 2. 键盘弹起状态：点击向下箭头，键盘收起
 * 除此之外，键盘的弹出和收起基于ui-input组件的focus和blur事件进行处理。
 */
var handleResizeForAndroid = {
  methods: {
    handleResizeForAndroid: function handleResizeForAndroid () {
      var this$1 = this;

      if (ui.IS_ANDROID) {
        window.addEventListener('resize', function () {
          var input = this$1.currentInput;
          if (input && !input.pageScrollByFocus && window.innerHeight < ui.WIN_HEIGHT) {
            androidInputScroll(input);
          }
        });
      }
    }
  }
};

/**
 * 页面生命周期mixin
 */
var STATUS = {
  LOADING: 0,
  SUCCESS: 1,
  ERROR: -1
};

function isDialog (vm) {
  var classList = Array.from(vm.$el.parentNode.classList);
  return classList.some(function (cls) {
    return cls.indexOf('ui-dialog-content') > -1
  })
}

function getTabPageClass (el) {
  while (Array.from(el.classList).indexOf('ui-tab-page') < 0) {
    if (el.parentNode && el.parentNode.classList) {
      el = el.parentNode;
    } else {
      break
    }
  }
  return getPageClass(el)
}

var lifeCycle = {
  mixins: [handleResizeForAndroid],
  data: function data () {
    return {
      // 页面配置
      pageConfig: {},
      // 页面加载状态，结合pageLoad使用
      STATUS: STATUS,
      status: STATUS.LOADING,
      // 是否初始化完成（也可用于是否首次进入页面）
      isInit: false,
      scrollType: null,
      pageClasses: [],
      insetBottom: 0
    }
  },
  created: function created () {
    this.setPageConfig();
  },
  mounted: function mounted () {
    var this$1 = this;

    var ref = this.pageConfig;
    var scrollType = ref.scrollType;
    var isTabBar = ref.isTabBar;
    var delay = ref.delay;

    this.isDialog = isDialog(this);
    this.delay = delay;
    this.container = scrollType === 'div' ? this.$refs.container : document.body;

    var currentClassList = Array.from(this.$el.classList);
    var classList = [];

    if (this.isDialog) {
      var dialogVm = this.$el.closest('.dialog-backdrop').__vue__;
      if (dialogVm) {
        classList.push(("ui-page-" + (dialogVm.scrollType) + "-scroll"));
      }
    } else {
      classList.push(("ui-page-" + (this.pageConfig.scrollType) + "-scroll"));
    }

    // 判定class是否包含/page-\w{6}/
    if (!isArryStringMatchPattern(currentClassList, /page-\w{6}/)) {
      if (isTabBar) {
        classList.push(getTabPageClass(this.$el));
      } else {
        classList.push(window.__$uiPageConfig.pageClass);
      }
    }

    if (process.env.NODE_ENV !== 'production' && window.navigator.userAgent.match(/windows/i)) {
      classList.push('windows-scrollbar');
    }

    this.pageClasses = classList;

    // **容器高度是固定的，内容高度不固定**
    if (scrollType === 'body') {
      this.height = window.innerHeight;
    } else {
      this.height = this.container.clientHeight;
    }

    // **tab页面的初始化由外部触发**
    if (!isTabBar || this.isDialog) {
      this.init();
    }

    this.$nextTick(function () {
      // 模拟onSafeAreaInsetsChange
      // window.setTimeout(() => {
      //   this.insetBottom = 34
      // }, 100)

      var insetBottom = ui.getSafeAreaInsetBottom();

      if (insetBottom === 34) {
        this$1.insetBottom = 26;
      } else {
        this$1.insetBottom = insetBottom;
      }

      ui.onSafeAreaInsetsChange(function () {
        var bottom = ui.getSafeAreaInsetBottom();
        if (bottom === 34) {
          this$1.insetBottom = 26;
        } else {
          this$1.insetBottom = bottom;
        }
      });

      var handleBack = function () {
        if (this$1.pageLoad && this$1.pageLoad.trigger === 'always') {
          this$1.handlePageLoad();
        }
      };
      if (ui.IS_APP && ui.on) {
        ui.on('UI_PAGE_BACK', handleBack);
      }
    });
  },
  activated: function activated () {
    if (this.isInit) {
      this.bindScroll();
      this.setBodyStyle();
      this.setStatusBarColor();

      if (this.pageLoad && this.pageLoad.trigger === 'always') {
        this.handlePageLoad();
      }
    }
  },
  deactivated: function deactivated () {
    this.restore();
  },
  beforeDestroy: function beforeDestroy () {
    this.restore();
  },
  methods: {
    // 处理页面的初始化，和handlePageLoad不同的是，此函数只执行一次
    init: function init () {
      var this$1 = this;

      var handle = function () {
        this$1.handlePageLoad()
          .then(function () {
            this$1.setBodyStyle();
            this$1.setStatusBarColor();
            this$1.bindScroll();
            // 使用$nextTick，避免初始化时执行activated的逻辑
            this$1.$nextTick(function () {
              this$1.transformContainer = this$1.$refs.content;
              this$1.initPull();
              this$1.isInit = true;
              this$1.handleResizeForAndroid();
            });
          });
      };

      if (ui.IS_APP && this.delay) {
        ui.onPageShow(handle);
      } else {
        handle();
      }

      // waitHandle(() => {
      //   this.handlePageLoad()
      //     .then(() => {
      //       this.transformContainer = this.$refs.content
      //       this.initPull()
      //
      //       this.setBodyStyle()
      //       this.setStatusBarColor()
      //       this.bindScroll()
      //       // 使用$nextTick，避免初始化时执行activated的逻辑
      //       this.$nextTick(() => {
      //         this.isInit = true
      //         this.handleResizeForAndroid()
      //       })
      //     })
      // }, this.delay)
    },

    /**
     * 处理页面加载
     * 1. pageLoad.handle为Promise函数，则显示loading状态
     * 2. 其他情况为加载成功
     */
    handlePageLoad: function handlePageLoad () {
      var this$1 = this;

      var promise = null;

      if (!this.pageLoad ||
        !this.pageLoad.handle ||
        !isPromise(promise = this.pageLoad.handle())) {
        this.status = STATUS.SUCCESS;
        return Promise.resolve(promise)
      }

      // 初始化时才显示loading
      if (!this.isInit) {
        this.status = STATUS.LOADING;
      }

      promise
        .then(function (data) {
          this$1.status = STATUS.SUCCESS;
        })
        .catch(function () {
          this$1.status = STATUS.ERROR;
        });

      return promise
    },
    /**
     * 外部触发tabLoad，只在tab-bar页面使用此方法
     */
    triggerTabLoad: function triggerTabLoad () {
      this.setPageConfig();

      this.setBodyStyle();

      if (this.isInit) {
        if (this.pageLoad && this.pageLoad.trigger === 'always') {
          this.handlePageLoad();
        }
        this.setStatusBarColor();
      } else {
        this.init();
      }
    },
    /**
     * 设置pageConfig和scrollType
     */
    setPageConfig: function setPageConfig () {
      this.pageConfig = getPageConfig();
      this.scrollType = this.pageConfig.scrollType;
    },
    /**
     * 还原页面状态
     */
    restore: function restore () {
      if (ui.IS_APP && ui.emit) {
        ui.emit('UI_PAGE_BACK');
      }
      this.unbindScroll();
      this.restoreBodyStyle();
      this.restoreStatusBarColor();
    }
  }
};

/**
 * 页面样式计算mixin
 */
var style = {
  computed: {
    styles: function styles () {
      var styles = {};

      var page = {};
      var content = {};

      var ref = this.pageConfig;
      var scrollType = ref.scrollType;
      var disableScroll = ref.disableScroll;
      var isTabBar = ref.isTabBar;
      var tabBarPosition = ref.tabBarPosition;
      var backgroundColor = ref.backgroundColor;

      var tabBarHeight = 55 + this.insetBottom;

      if (isTabBar) {
        if (backgroundColor) {
          page.backgroundColor = backgroundColor;
        } else {
          page.backgroundColor = null;
        }
      }

      var top = 0;
      var bottom = 0;
      var contentPaddingBottom = 0;
      // minHeight处理
      // iOS: minHeight = page高度 + 0.5 (出弹性滚动)
      // Android: minHeight = page高度
      var minHeight = ui.IS_IOS && !this.isDialog ? window.innerHeight + 0.5 : window.innerHeight;

      if (this.top !== undefined) {
        top = Number(this.top);
      } else {
        top += ui.DEFAULT_HEADER_HEIGHT;
      }

      if (isTabBar) {
        if (tabBarPosition === 'top') {
          top += tabBarHeight;
        } else {
          bottom += tabBarHeight;

          if (scrollType === 'body') {
            contentPaddingBottom += tabBarHeight;
          }
        }
      }

      // 当作为dialog使用时
      if (this.isDialog) {
        bottom = 0;

        if (this.top !== undefined) {
          minHeight -= Number(this.top);
        } else {
          minHeight -= ui.DEFAULT_HEADER_HEIGHT;
        }
      } else {
        minHeight += -(top + bottom);
      }

      // 整页滚动时设置page的paddingTop, paddingBottom
      if (scrollType === 'body' || this.isDialog) {
        page.paddingTop = top + 'px';
        page.paddingBottom = bottom + 'px';
      } else {
        // 区域滚动时设置page的top, bottom
        page.top = top + 'px';
        page.bottom = bottom + 'px';
        if (disableScroll) {
          page.overflowY = 'hidden';
          page.webkitOverflowScrolling = 'auto';
          document.body.style.overflowY = null;
        }
      }

      if (this.enableReachBottomLoad) {
        contentPaddingBottom += 50;
      }

      // dialog不需要paddingBottom
      if (this.isDialog) {
        contentPaddingBottom = 0;
      }

      content.paddingBottom = contentPaddingBottom ? contentPaddingBottom + 'px' : null;

      if (scrollType === 'div') {
        content.minHeight = minHeight + 'px';
        this.contentHeight = minHeight;
      }

      content = Object.assign({}, content, this.contentStyle);

      styles.page = page;
      styles.content = content;

      return styles
    }
  }
};

/**
 * document.body样式计算mixin
 */
var bodyStyle = {
  methods: {
    setBodyStyle: function setBodyStyle () {
      var ref = this.pageConfig;
      var isTabBar = ref.isTabBar;
      var disableScroll = ref.disableScroll;
      var backgroundColor = ref.backgroundColor;
      var backgroundTextStyle = ref.backgroundTextStyle;
      var scrollType = ref.scrollType;
      var showScroller = ref.showScroller;
      var title = ref.title;

      this.disableScroll = disableScroll;

      if (!isTabBar) {
        document.body.style.backgroundColor = backgroundColor;
      }
      document.body.style.color = backgroundTextStyle === 'light' ? '#ccc' : '#333';

      if (scrollType === 'body') {
        if (showScroller) {
          ui.showScrollIndicator();
        }
        document.body.style.overflowY = disableScroll ? 'hidden' : null;
      } else {
        document.body.style.overflowY = 'hidden';
      }
      document.title = title || window.__$uiTitle;
    },
    restoreBodyStyle: function restoreBodyStyle () {
      var ref = this.pageConfig;
      var isTabBar = ref.isTabBar;
      var disableScroll = ref.disableScroll;
      var scrollType = ref.scrollType;
      var showScroller = ref.showScroller;

      this.disableScroll = disableScroll;

      if (!isTabBar) {
        document.body.style.backgroundColor = null;
      }
      document.body.style.color = null;

      if (scrollType === 'body') {
        if (showScroller) {
          ui.hideScrollIndicator();
        }
        if (disableScroll) {
          document.body.style.overflowY = null;
        }
      } else {
        document.body.style.overflowY = null;
      }
    }
  }
};

/**
 * 页面滚动处理mixin
 */
var scroll = {
  methods: {
    bindScroll: function bindScroll () {
      /**
       * 监听scroll事件
       * 区域滚动
       *  - 监听container滚动事件
       * 全屏滚动
       *  - 在web下监听window滚动事件
       *  - 在app下监听ui.onScroll事件
       */
      this.__initScrollEvent();
      if (this.scrollType === 'div') {
        this.container.addEventListener('scroll', this.__divScroll);
      } else {
        window.addEventListener('scroll', this.__bodyScroll);
      }

      // 处理初始滚动位置
      if (this.scrollTop) {
        this.scrollTo(this.scrollTop);
      }

      /**
       * 监听iOS返回顶部
       */
      if (this.enableBackToTop) {
        ui.onStatusbarTap(this.backToTop);
      }
    },
    unbindScroll: function unbindScroll () {
      if (this.scrollType === 'div') {
        this.container.removeEventListener('scroll', this.__divScroll);
      } else {
        window.removeEventListener('scroll', this.__bodyScroll);
      }
    },
    __initScrollEvent: function __initScrollEvent () {
      var this$1 = this;

      this.__divScroll = function () {
        this$1.scrollPosition = this$1.container.scrollTop;
        this$1.__emitScrollEvents();
      };

      this.__bodyScroll = function () {
        // Chrome 61不认document.body.scrollTop，一直都是0
        // Safari不认document.documentElement.scrollTop，一直都是0
        this$1.scrollPosition = document.body.scrollTop || document.documentElement.scrollTop;

        this$1.__emitScrollEvents();
      };
    },
    __emitScrollEvents: function __emitScrollEvents () {
      if (this.scrollState === 0) {
        this.scrollState = 1;
        this.$emit('scrollstart', this.scrollPosition);
      }

      if (this.scrollState === 1) {
        this.$emit('scroll', this.scrollPosition);
      }

      this.__detectScrollEnd();
    },
    __detectScrollEnd: function __detectScrollEnd () {
      var this$1 = this;

      window.clearTimeout(this.scrollEndTimeout);
      this.scrollEndTimeout = window.setTimeout(function () {
        this$1.scrollState = 0;
        this$1.$emit('scrollend', this$1.scrollPosition);

        if (this$1.handleInnerScrollEnd) {
          this$1.handleInnerScrollEnd();
        }
      }, this.scrollEndDelay);
    },
    /**
     * 返回顶部
     */
    backToTop: function backToTop () {
      this.__scroll(0, 500);
    },
    /**
     * 滚动到指定位置或元素
     */
    scrollTo: function scrollTo (target, time) {
      if ( time === void 0 ) time = 500;

      if (/^\d+$/.test(target)) {
        this.__scroll(target, time);
      } else if (/^#.*/.test(target)) {
        var el = document.querySelector(target);
        var offsetTop = getRelativeOffset(el, this.container)['scrollTop'];
        this.__scroll(offsetTop, time);
      }
    },
    /**
     * 动画滚动
     */
    __scroll: function __scroll (target, speed) {
      if ( target === void 0 ) target = 0;
      if ( speed === void 0 ) speed = 0;

      var container = this.container;
      container.style.webkitOverflowScrolling = 'auto';
      animateScroll({
        el: container,
        scrollTarget: target,
        speed: speed
      });
      window.setTimeout(function () {
        container.style.webkitOverflowScrolling = 'touch';
      }, speed);
    }
  }
};

/**
 * app状态栏设置mixin
 */
var statusBar = {
  data: function data () {
    return {
      statusBarColor: null
    }
  },
  methods: {
    // 设置状态栏字体颜色
    setStatusBarColor: function setStatusBarColor () {
      var this$1 = this;

      if (ui.IS_APP && this.pageConfig.navigationBarTextStyle && !this.isDialog) {
        ui.getStatusBarStyle({
          // 使用箭头函数，保留this引用
          success: function (data) {
            this$1.statusBarColor = data.style;
          }
        });

        ui.setStatusBarStyle({
          style: this.pageConfig.navigationBarTextStyle
        });

        // 访问tabBar的第一个页面时，记住当前的状态栏颜色
        if (this.pageConfig.isTabBar && !window.__$uiRemStatusColor) {
          window.__$uiRemStatusColor = this.statusBarColor;
        }
      }
    },
    // 还原状态栏字体颜色
    restoreStatusBarColor: function restoreStatusBarColor () {
      if (ui.IS_APP && this.statusBarColor && !this.isDialog) {
        if (this.pageConfig.isTabBar) {
          // tabBar页面后退时，对处于活动状态的页面执行重置，其他tabBar页面不重置
          if (this.isTabBarActive) {
            var statusBarColor = window.__$uiRemStatusColor || this.statusBarColor;
            window.__$uiRemStatusColor = '';
            ui.setStatusBarStyle({
              style: statusBarColor
            });
          }
        } else {
          ui.setStatusBarStyle({
            style: this.statusBarColor
          });
        }
      }
    }
  }
};

var Mixins = {
  mixins: [lifeCycle, style, bodyStyle, scroll, statusBar]
};

var index = {render: function(){var _vm=this;var _h=_vm.$createElement;var _c=_vm._self._c||_h;return _c('div',{ref:"container",staticClass:"ui-page",class:_vm.pageClasses,style:(_vm.styles.page),on:{"click":_vm.handlePageClick}},[(_vm.$slots['nav-bar'])?_c('transfer-dom',{directives:[{name:"show",rawName:"v-show",value:(_vm.selfShowCustomNavBar),expression:"selfShowCustomNavBar"}],style:(_vm.transformNavBarStyle)},[_vm._t("nav-bar")],2):_vm._e(),_vm._v(" "),(_vm.selfShowLoading)?_vm._t("loading",[(_vm.loadingTemplate)?_c('div',{staticClass:"ui-page-loading",style:({ top:(this.contentHeight / 2 -30)  + 'px' }),domProps:{"innerHTML":_vm._s(_vm.loadingTemplate)}}):_c('div',{staticClass:"ui-page-loading",style:({ top:(this.contentHeight / 2 -30)  + 'px' })},[_vm._m(0)])]):_vm._e(),_vm._v(" "),(_vm.status === _vm.STATUS.ERROR)?_vm._t("retry",[_c('div',{staticClass:"retry"},[_vm._v("轻触重试...")])]):_vm._e(),_vm._v(" "),(_vm.status === _vm.STATUS.SUCCESS)?_c('div',{ref:"content",staticClass:"ui-page-content",class:{ 'disable-scroll': _vm.disableScroll },style:(_vm.styles.content)},[(_vm.pullDown)?_vm._t("pull-down",[_c('pull-hook',{attrs:{"pull-style":_vm.pullDownStyle,"state":_vm.pullState,"translate":_vm.pullTranslate,"type":"pull-down"}})]):_vm._e(),_vm._v(" "),_vm._t("default"),_vm._v(" "),(_vm.pullUp)?_vm._t("pull-up",[_c('pull-hook',{attrs:{"pull-style":_vm.pullUpStyle,"state":_vm.pullState,"translate":_vm.pullTranslate,"type":"pull-up"}})]):_vm._e(),_vm._v(" "),(_vm.enableReachBottomLoad && _vm.showLoadMore)?_vm._t("load-more",[_vm._m(1)]):_vm._e()],2):_vm._e()],2)},staticRenderFns: [function(){var _vm=this;var _h=_vm.$createElement;var _c=_vm._self._c||_h;return _c('div',{staticStyle:{"position":"relative","width":"34px","height":"34px","margin":"0 auto"}},[_c('div',{staticClass:"ui-page-loading-spinner ui-page-loading-spinner-max"},[_c('div',{staticClass:"ui-page-loading-spinner ui-page-loading-spinner-mid"},[_c('div',{staticClass:"ui-page-loading-spinner ui-page-loading-spinner-min"})])]),_vm._v(" "),_c('div',{staticClass:"ui-page-loading-timer"},[_vm._v("Loading…")])])},function(){var _vm=this;var _h=_vm.$createElement;var _c=_vm._self._c||_h;return _c('div',{staticClass:"load-more-content"},[_c('div',{staticClass:"load-more-icon loading"}),_vm._v(" "),_c('span',[_vm._v("正在加载...")])])}],
  name: 'UiPage',
  mixins: [Mixins],
  props: {
    enableAndroidPullDown: Boolean,
    enableReachBottomLoad: Boolean,
    // 滚动到底加载触发距离
    reachBottomLoadDistance: {
      type: Number,
      default: 0
    },
    pullDown: Object,
    pullUp: Object,
    onLoadMore: {
      type: Function,
      default: function () {}
    },
    // 页面加载
    pageLoad: {
      type: Object
    },
    // top定位
    top: [Number, String],
    // 滚动位置
    scrollTop: {
      type: [Number, String],
      default: 0
    },
    // 内容样式
    contentStyle: Object,
    // 是否启用回到顶部功能
    enableBackToTop: {
      type: Boolean,
      default: true
    },
    showLoading: {
      type: Boolean,
      default: true
    },
    autoHideLoading: {
      type: Boolean,
      default: true
    }
  },
  components: {
    Spinner: Spinner,
    TransferDom: TransferDom,
    PullHook: PullHook
  },
  data: function data () {
    var scene = '';
    if (ui.IS_APP) {
      scene = ui.IS_IOS ? 'ios' : 'android';
    } else {
      scene = 'web';
    }

    return {
      pullState: 0,
      pullTranslate: 0,
      // 到容器顶部的距离
      scrollPosition: 0,
      // 是否显示“加载更多”
      showLoadMore: false,
      // 容器高度
      height: 0,
      // 内容高度
      contentHeight: ui.DEFAULT_CONTENT_HEIGHT,
      // 滚动事件截流时间
      throttleTime: 0,
      // 滚动状态
      scrollState: 0,
      // 是否显示自定义nav-bar（用于tab-bar切换时）
      showCustomNavBar: true,
      // 检测scrollend的定时器
      scrollEndTimeout: null,
      // 检测scrollend的时间
      scrollEndDelay: 100,
      // 是否为dialog页面
      isDialog: false,
      // 作为dialog页面时，dialog的显示状态
      isDialogShow: false,
      // android, ios, web
      scene: scene,
      loadingTemplate: window.__$uiTemplate.loading || '',
      isTabBarActive: false,
      disableScroll: false
    }
  },
  computed: {
    transformNavBarStyle: function transformNavBarStyle () {
      var style = {};
      if (this.isDialog) {
        style.position = 'fixed';
        style.top = 0;
        style.left = 0;
        style.zIndex = 505;
      }
      return style
    },
    selfShowCustomNavBar: function selfShowCustomNavBar () {
      return this.isDialog ? this.isDialogShow : this.showCustomNavBar
    },
    selfShowLoading: function selfShowLoading () {
      if (!this.showLoading) {
        return false
      }

      if (this.showLoading && !this.autoHideLoading) {
        return true
      }

      return this.status === this.STATUS.LOADING
    },
    // pageClasses () {

    //   let classList = ['ui-page', `ui-page-${this.pageConfig.scrollType}-scroll`, window.__$uiPageConfig.pageClass]
    //   if (process.env.NODE_ENV !== 'production' && window.navigator.userAgent.match(/windows/i)) {
    //     classList.push('windows-scrollbar')
    //   }
    //   return classList
    // },
    pullDownStyle: function pullDownStyle () {
      return this.pullDown.disableContentMove &&
      this.pullDown.disableContentMove.indexOf(this.scene) > -1 &&
      !this.$slots['pull-down']
      ? 'android'
      : 'ios'
    },
    pullUpStyle: function pullUpStyle () {
      return this.pullUp.disableContentMove &&
      this.pullUp.disableContentMove.indexOf(this.scene) > -1 &&
      !this.$slots['pull-up']
      ? 'android'
      : 'ios'
    }
  },
  watch: {
    scrollPosition: function scrollPosition (val) {
      if (this.container) {
        var totalDistance = val + window.innerHeight + this.reachBottomLoadDistance;
        var height = this.container.scrollHeight;

        this.showLoadMore = totalDistance >= height;
      }
    },
    scrollTop: function scrollTop (val) {
      if (val >= 0) {
        this.scrollTo(val);
      }
    },
    showLoadMore: function showLoadMore (val) {
      if (val && this.enableReachBottomLoad) {
        this.onLoadMore();
      }
    }
  },
  methods: {
    initPull: function initPull () {
      this.pull = PullFactory.create(this);
    },
    handlePullStateChange: function handlePullStateChange (state) {
      this.pullState = state;
    },
    handlePullMove: function handlePullMove (translate, state) {
      this.pullTranslate = translate;
      this.pullState = state;
    },
    triggerPullDown: function triggerPullDown () {
      this.pull.triggerPullDown();
    },
    triggerPullUp: function triggerPullUp () {
      this.pull.triggerPullUp();
    },
    handlePageClick: function handlePageClick () {

    }
  }
};

export default index;
